
    SPAWN_MDL_CLASS     PROTO

; 
;
; CLASS: MDL - Module base class
;
; METHODS:  Reset, Run
;
    MDL_CONSTRUCTOR     PROTO
    MDL_DESTRUCTOR      PROTO

    MDL_RESET           PROTO
    MDL_RUN             PROTO
    MDL_CONNECT         PROTO :DWORD,:DWORD
    MDL_GETOUTPUT       PROTO :DWORD
    MDL_DISCONNECT      PROTO :DWORD
    MDL_STA             PROTO
;
; PROPERTIES: smp
;
; 

ifndef _MDL_

    _MDL_ EQU 1000h


; 
;
; Error and other message codes
;
; -------------------------------------------------------------------------

  MDL_EC_OK                     EQU COMMON_EC_OK
  MDL_EC_FAIL                   EQU COMMON_EC_FAIL
  MDL_EC_ILLEGAL_OPERATION      EQU COMMON_EC_ILL_OP
  MDL_EC_ALLOC_ERROR            EQU COMMON_EC_ALLOC_ERROR
  MDL_EC_DEALLOC_ERROR          EQU COMMON_EC_DEALLOC_ERROR

; 
; 



; 

MDL_CLASS   STRUCT

; Methods
            Constructor dd  ?           ; constructor
            Destructor  dd  ?           ; destructor
; Virtual Methods
            Reset       dd  ?           ; virtual function
            Run         dd  ?           ; virtual function
            Connect     dd  ?           ; virtual function
            Disconnect  dd  ?           ; virtual function
            GetOutput   dd  ?           ; virtual function
            Statistic   dd  ?           ; virtual function
; Properties
            smp      real8 ?           ; holds actual sample value
            parent      dd  ?           ; pointer to parent object

MDL_CLASS   ENDS

; 
; Forward reference

      NET_CLASS   STRUCT
      
                               MDL_CLASS <>       ; base class
      ; Properties
                  ModuleList          dd ?        ; pointer to admin list
                                                  ; struct of admin list
                                                  ;   0000  DWORD - count of enrties
                                                  ;   0004- array of STRUCT NET_mLIST_ELEM
                  ModuleSpaceStart    dd ?        ; starting address of module space
                  ModuleSpaceCurr     dd ?        ; current pointer
      
                  DumyOutput          dd ?        ; output for module inputs not connected (default value)
      
      NET_CLASS   ENDS

; 

GATE        STRUCT

            input       dd ?        ; pointer to an output
            value    REAL8 ?        ; copy of output for caching

GATE        ENDS
; 
;
; RESET_GATE - resets the gate pointer, cleares data
;
; IN:   Gate - gate to be reset
;
; Out:  
;
; 

RESET_GATE  MACRO Gate:REQ

            mov     eax,[esi].parent
            add     eax,NET_CLASS.DumyOutput
            mov     [esi].Gate.input,eax
            xor     eax,eax
            mov     DWORD PTR [esi].Gate.value,eax
            mov     DWORD PTR [esi].Gate.value+4,eax

            ENDM

; 
;
; READ_GATE - reads the input and stores in the cache
;
; IN:   Gate - gate to be read
;
; Out:  
;
; 

READ_GATE   MACRO Gate:REQ
            
            mov     edi,[esi].Gate.input
            mov     eax,DWORD PTR [edi]
            mov     eax,DWORD PTR [edi]+4
            mov     DWORD PTR [esi].Gate.value,eax
            mov     DWORD PTR [esi].Gate.value+4,eax

            ENDM

; 
;
; UPDATE_GATE - reads the input and checks for change
;
; IN:   Gate - gate to be read
;
; Out:  ZF=1 - no change
;       ZF=0 - value changed
;
; 

UPDATE_GATE MACRO Gate:REQ

            mov     edi,[esi].Gate.input
        fld REAL4 PTR [esi].Gate.value
        fcomp [edi]
        fstsw ax
        sahf

            ENDM

; 


endif