    .586                       ; create 32 bit code
    .model flat, stdcall       ; 32 bit memory model
    option casemap :none       ; case sensitive


    include \masm32\include\kernel32.inc
    include \masm32\include\windows.inc
    include \masm32\macros\macros.asm

    include \masm32\include\masm32.inc
    include \masm32\include\debug.inc

    include network.inc

    AddToLog        PROTO :DWORD

; 

NET_DEF_NUMBER_OF_ENTRIES   EQU 00FFh
NET_DEF_MOD_SPACE_SIZE      EQU NET_DEF_NUMBER_OF_ENTRIES*0080h

; 
;
; Structure for internal administration of modules
;
; 

NET_INT_ELEM        STRUCT
                    int_type    dd 0                ; module type
                    int_size    dd 0                ; size of module
                    int_name    db 16 dup (0)       ; module name
                    int_spawn   dd 0                ; initialize
NET_INT_ELEM        ENDS

; 

 .data
align 4
; 

    NET_Module_Table    NET_INT_ELEM <_CONST,sizeof(INPUT_CLASS), \
                                      'constant', \
                                      offset SPAWN_INPUT_CLASS>
                        NET_INT_ELEM <_POT,sizeof(POT_CLASS), \
                                      'potmeter', \
                                      offset SPAWN_POT_CLASS>

                        NET_INT_ELEM <_OSC_IMP,sizeof(OSC1_CLASS), \
                                      'impulse osc', \
                                      offset SPAWN_OSC1_CLASS>
                        NET_INT_ELEM <_OSC_SAW,sizeof(OSC1_CLASS), \
                                      'saw osc', \
                                      offset SPAWN_OSC1_CLASS>
                        NET_INT_ELEM <_OSC_RND,sizeof(OSC1_CLASS), \
                                      'noise osc', \
                                      offset SPAWN_OSC1_CLASS>
                        NET_INT_ELEM <_OSC_SIN,sizeof(OSC1_CLASS), \
                                      'sinus osc', \
                                      offset SPAWN_OSC1_CLASS>

                        NET_INT_ELEM <_OSC_PLS,sizeof(OSC2_CLASS), \
                                      'pulse osc', \
                                      offset SPAWN_OSC2_CLASS>
                        NET_INT_ELEM <_OSC_PSAW,sizeof(OSC2_CLASS), \
                                      'pulse saw osc', \
                                      offset SPAWN_OSC2_CLASS>
                        NET_INT_ELEM <_OSC_TRI,sizeof(OSC2_CLASS), \
                                      'triangle osc', \
                                      offset SPAWN_OSC2_CLASS>

                        NET_INT_ELEM <NULL,0,' empty ',0>

; 
;
; Messages
;                       01230123012301230123012301230123
    NET_M_CREATE    db 'Network module created.',0Dh,0Ah,0
    NET_M_NAME      db 'Network',0
    NET_M_MODCOUNT  db ' 000 module(s) created.',0Dh,0Ah,0
    NET_M_NOMODULES db '  No modules.',0Dh,0Ah,0
    NET_M_SEQNO     db '000.',0
    NET_M_DESTROY   db 'Network module destroyed.',0
    NET_M_CRLF      db 0Dh,0Ah,0


; 
; 
;
; IMPORTANT!!!
;
; EVERY METHOD HAS AN IMPLICIT PARAMETER: esi - PTR NET_CLASS
;
; UNPRESERVED REGISTERS: eax
;
; 
; 

 .code

; 
;
; This PROC must be executed after the necessary space for a new instant of
; NET_CLASS class was allocated. Register esi holds the instance pointer.
;
; Loads the pointers of the methods with the appropriate values
;
; 
SPAWN_NET_CLASS PROC USES ebx esi
                ASSUME eax:PTR NET_CLASS

                lea     ebx,NET_CONSTRUCTOR
                mov     [eax].Constructor,ebx
                lea     ebx,NET_DESTRUCTOR
                mov     [eax].Destructor,ebx

; virtual functions
                ;lea     ebx,NET_RESET
                ;mov     [eax].Reset,ebx
                lea     ebx,NET_RUN
                mov     [eax].Run,ebx
                lea     ebx,NET_STA
                mov     [eax].Statistic,ebx

                mov     esi,eax
                invoke NET_CONSTRUCTOR

                ret

                ASSUME eax:NOTHING
SPAWN_NET_CLASS ENDP

; 
;
; Constructor
;
; 1. Init the properties
; 2. Allocate admin space
; 3. Allocate space for modules
;
; 
NET_CONSTRUCTOR PROC USES ebx
align 4
LOCAL   ErrorCode:DWORD

                ASSUME esi:PTR NET_CLASS

@_1: ; initialize properties
                xor     eax,eax                     
                mov     [esi].DumyOutput,eax            ; dumy output is zero
                mov     [esi].parent,eax                ; no parent object

@_2: ; allocate space for admin list
                invoke GlobalAlloc,GPTR, \
                        NET_DEF_NUMBER_OF_ENTRIES*sizeof NET_mLIST_ELEM
                mov     ErrorCode,NET_EC_ALLOC_ERROR
                or      eax,eax
                jz  @_END

                mov     [esi].ModuleList,eax            ; list is allocated
                mov     [eax],DWORD PTR 0

@_3: ; allocate space for modules
                invoke GlobalAlloc,GPTR,NET_DEF_MOD_SPACE_SIZE
                mov     ErrorCode,NET_EC_ALLOC_ERROR
                or      eax,eax
                jz  @_END
                mov     [esi].ModuleSpaceStart,eax      ; space is allocated
                mov     [esi].ModuleSpaceCurr,eax

                mov     ErrorCode,NET_EC_OK

                invoke MDL_CONSTRUCTOR
                invoke AddToLog,addr NET_M_CREATE
@_END:
                mov     eax,ErrorCode
                ret
                ASSUME esi:NOTHING
NET_CONSTRUCTOR ENDP
; 
;
; Destructor
;
; 1. Remove every module
; 2. Deallocate admin space
;
; 
NET_DESTRUCTOR  PROC USES ecx edi
align 4

                ASSUME esi:PTR NET_CLASS

@_1: ; remove every module
                mov     edi,[esi].ModuleList
                or      edi,edi                         ; list exists?
                jz  @_20

                mov     ecx,[edi]                       ; count of entries
                or      ecx,ecx
                jz  @_2

                push    esi
                ASSUME edi:PTR NET_mLIST_ELEM
                add     edi,4
@_10:
                mov     esi,[edi].mAddr
                call DWORD PTR [esi].Destructor         ; destroy module

                add     edi,sizeof(NET_mLIST_ELEM)
                sub     ecx,1
align 4
                jnz  @_10
                ASSUME edi:NOTHING
                pop     esi

@_2: ; deallocate admin space
                invoke GlobalFree,[esi].ModuleList
                or      eax,eax                         ; GlobalFree failed?
                jz  @_20
                mov     eax,NET_EC_DEALLOC_ERROR
                jmp @_END
@_20:
                invoke GlobalFree,[esi].ModuleSpaceStart
                or      eax,eax                         ; HeapFree failed?
                jz  @_21
                mov     eax,NET_EC_DEALLOC_ERROR
                jmp @_END
@_21:                
                mov     eax,NET_EC_OK

                invoke AddToLog,addr NET_M_DESTROY
@_END:
                ret
                ASSUME esi:NOTHING
NET_DESTRUCTOR  ENDP
; 
;
; Method - Run
;
; 
NET_RUN         PROC
                ASSUME esi:PTR NET_CLASS

                ret
                ASSUME esi:NOTHING
NET_RUN         ENDP
; 
;
; Method - Statistic
;
; 
NET_STA         PROC USES ebx ecx
                ASSUME esi:PTR NET_CLASS

                invoke AddToLog,addr NET_M_CRLF
                invoke AddToLog,addr NET_M_NAME
                invoke MDL_STA

                mov     edi,[esi].ModuleList
                mov     ecx,[edi]               ; count

                or      ecx,ecx
                jnz  @_1
                invoke AddToLog,addr NET_M_NOMODULES
                ret
@_1:
                xor     ebx,ebx
                invoke DW2STR,ecx,addr NET_M_MODCOUNT+1,10,3
                invoke AddToLog,addr NET_M_MODCOUNT
                invoke AddToLog,addr NET_M_CRLF
                push    esi
                add     edi,4
                ASSUME edi:PTR NET_mLIST_ELEM
@_10:
                add     ebx,1
                invoke DW2STR,ebx,addr NET_M_SEQNO,10,3
                invoke AddToLog,addr NET_M_SEQNO
                mov     eax,[edi].mType
                lea     esi,[NET_Module_Table]
                ASSUME esi:PTR NET_INT_ELEM
align 4
@_11:
                cmp     eax,[esi].int_type
                jz  @_12
                add     esi,sizeof(NET_INT_ELEM)
                jmp @_11
@_12:
                add     esi,NET_INT_ELEM.int_name
                invoke AddToLog,esi
                ASSUME esi:PTR MDL_CLASS

                mov     esi,[edi].mAddr
                call [esi].Statistic
                add     edi,sizeof NET_mLIST_ELEM
                invoke AddToLog,addr NET_M_CRLF
                sub     ecx,1
                jnz @_10
                pop     esi
                ASSUME esi:PTR NET_CLASS
@_2:
                ret
                ASSUME esi:NOTHING
NET_STA         ENDP
; 
;
; NewMdl - create new modul
;
; 1. Determine the type by the code
; 2. Allocate space and init module
; 3. Insert new instant into admin list (type,address)
; 4. Run constructor of module
;
; IN:   mtype - type of module
;
; OUT:  eax=addr - on success
;       eax=NULL - on fail
;
; 
NET_NewMdl      PROC USES ebx ecx esi edi t:DWORD
LOCAL   MdlAddr:DWORD

                ASSUME esi:PTR NET_CLASS

                lea     edi,[NET_Module_Table]
                ASSUME edi:PTR NET_INT_ELEM

align 4
@_1: ; determine type, get info
                mov     eax,[edi].int_type
                or      eax,eax
                jz  @_END
                cmp     eax,t
                jz  @_2
                add     edi,sizeof(NET_INT_ELEM)
                jmp @_1

@_2: ; allocate space
                mov     eax,[esi].ModuleSpaceCurr
    ; TODO: is there enough space?
                mov     ecx,[edi].int_size
                mov     ebx,eax
                add     [esi].ModuleSpaceCurr,ecx

                push    esi
                xchg    eax,esi
                mov     [esi].parent,eax            ;parent obj is 'this'
                ASSUME esi:PTR MDL_CLASS
                mov     eax,t
                call DWORD PTR [edi].int_spawn
                pop     esi
                ASSUME esi:PTR NET_CLASS

                ASSUME edi:NOTHING
@_3: ; insert into admin list
                mov     edi,[esi].ModuleList
                mov     eax,[edi]                   ; count of entries
                push    eax
                add     eax,1
                mov     [edi],eax
                pop     eax
                add     edi,4
                ASSUME edi:PTR NET_mLIST_ELEM
                shl     eax,4                       ; *sizeof(NET_mLIST_ELEM)
                add     edi,eax
                mov     eax,t
                mov     [edi].mType,eax
                mov     [edi].mAddr,ebx
                mov     [edi].mSize,ecx
                ASSUME ebx:NOTHING

                ASSUME esi:PTR NET_CLASS
@_END:
                ret
                ASSUME edi:NOTHING
                ASSUME esi:NOTHING
NET_NewMdl      ENDP
; 
;
; FreeMdl - destroy modul
;
; 1. Get module data from list
; 2. Disconnect module from network
; 3. Run destructor of module
; 4. Deallocate space
; 5. Remove instant from admin list
;
; IN:   id - id of module (id in admin list)
;
; OUT:  esi=NULL - on success
;       esi=addr - on fail
; 
; 
NET_FreeMdl     PROC USES ebx ecx edi esi id:DWORD
align 4
LOCAL MyHeap:DWORD
LOCAL ErrorCode:DWORD
LOCAL mCount:DWORD
                ASSUME esi:PTR NET_CLASS

@_1: ; get adminlist entry of module
                mov     eax,id
                mov     edi,[esi].ModuleList
                mov     ErrorCode,NET_EC_ILLEGAL_OPERATION
                or      edi,edi                     ; module list empty error
                jz  @_END
                mov     ecx,[edi]                   ; count of entries
                mov     mCount,ecx
                add     edi,4
                or      ecx,ecx
                jz  @_END
                shl     eax,4                       ; *sizeof NET_mLIST_ELEM
                add     edi,eax                     ; get list entry address

@_2: ; disconnect module
                push    esi
                mov     eax,edi

                ASSUME edi:PTR NET_mLIST_ELEM
                push    edi
                mov     edi,[esi].ModuleList
                mov     ecx,mCount
@_20:
                mov     esi,[edi].mAddr             ; get module
                ASSUME esi:PTR MDL_CLASS
                push    eax
                call [esi].Disconnect               ; call disconnect method
                sub     ecx,1
                jnz @_20
                pop edi

@_3: ; run destructor of module

                mov     esi,[edi].mAddr             ; address of module
                call DWORD PTR [esi+4]              ; call DESTRUCTOR

                pop     esi


                ASSUME edi:NOTHING
                
                invoke HeapFree,
                        MyHeap,
                        NULL,
                        ebx
                mov     ErrorCode,NET_EC_OK
@_END:
                mov     eax,ErrorCode
                ret

                ASSUME esi:NOTHING
NET_FreeMdl     ENDP
; 
;
; GetMdlByAdr - Get data of module from admin list selected by address
;
; IN:   address - possible address of a module
;
; OUT:  on success
;           eax - index
;           ebx - address
;
;       on fail
;           eax - NULL
;
; 
NET_GetMdlByAdr PROC USES ecx edx edi adr:DWORD
align 4
LOCAL count:DWORD

                ASSUME esi:PTR NET_CLASS

                xor     eax,eax
                mov     edi,[esi].ModuleList
                or      edi,edi
                jz  @_END
                mov     ecx,[edi]
                or      ecx,ecx
                jz  @_END
                mov     count,ecx
                add     edi,4
                mov     edx,adr

                xor     ecx,ecx
                ASSUME  edi:PTR NET_mLIST_ELEM
@_1:
                mov     eax,[edi].mAddr         ;start address
                cmp     edx,eax                 ;check low boundary
                jc  @_2
                mov     ebx,eax
                add     eax,[edi].mSize
                cmp     edx,eax                 ;check high boundary
                jc  @_END
@_11:                
                add     edi,sizeof NET_mLIST_ELEM
                add     ecx,1
                cmp     ecx,count
                jc  @_1
                ASSUME  edi:NOTHING
@_2:
                xor     eax,eax
                ret
@_END:
                mov     eax,ecx
                add     eax,1

                ret
                ASSUME esi:NOTHING
NET_GetMdlByAdr ENDP

; 
;
; GetMdlByType - Get data of module from admin list selected by type
;
; IN:   mtype - type of module
;
; OUT:  on success
;           esi - address
;           ebx - index
;
;       on fail
;           esi - NULL
;
; 
NET_GetMdlByType PROC mtype:DWORD,flag:DWORD
align 4
                ASSUME esi:PTR NET_CLASS



                ret
                ASSUME esi:NOTHING
NET_GetMdlByType ENDP

; 
;
; METHOD: Build network from binary data
;
; 
NET_BuildFromBin    PROC lpData:DWORD
align 4
                    ASSUME esi:PTR NET_CLASS

                    ret
                    ASSUME esi:NOTHING
NET_BuildFromBin    ENDP
; 
;
; METHOD: Export network in binary form
;
; 
NET_ExportBin       PROC lpData:DWORD
align 4
                    ASSUME esi:PTR NET_CLASS

                    ret
                    ASSUME esi:NOTHING
NET_ExportBin       ENDP
; 
;
; METHOD: Build network
;
;  IN:      lpData - pointer to area containing network description
;
;           0000        dd number of modules
;           0004-xxxx   dd module codes
;           xxxx+4-     dd first module's data:
;                 00    db first connected input's (gate) seq.number
;               0000    db connected module's seq.number
;                 00    db connected module's output's seq.number
;
; For constants their value are stored. smp is the 0. output
;
; Example:
;   Modules: const1=110.0, const2=250.0, const3=0.6, osc1=saw, osc2=sin
;   Connections: osc1.fre=const1, osc2.fre=const2, osc1.amp=const3, osc2.amp=const3
;
;           0000h        dd 05
;           0000h-0013h  dd _CONST,_CONST,_CONST,_OSC_SAW,_OSC_SIN
;           0014h-0018h  dd(=REAL4) 110.0
;           0018h-001Bh  dd(=REAL4) 250.0
;           001Ch-001Fh  dd(=REAL4) 0.6
;           0020h        db 2(=fre)
;           0021h        dw 1
;           0023h        db 0
;           0024h        db 1(=amp)
;           0025h        dw 3
;           0027h        db 0
;           0028h        db 2(=fre)
;           0029h        dw 2
;           002Bh        db 0
;           002Ch        db 1(=amp)
;           002Dh        dw 3
;           002Fh        db 0
;           0030h        dd 0   - end
;
; 
NET_Build           PROC USES ebx ecx edx edi lpData:DWORD
align 4
                    ASSUME esi:PTR NET_CLASS

                    mov     edi,lpData
                    mov     ecx,[edi]
                    add     edi,4
@_1:
                    mov     eax,[edi]
                    push    eax
                    call  NET_NewMdl
                    or      eax,eax
                    jz  @_END
                    add     edi,4
                    sub     ecx,1
                    jnz @_1

                    mov     edx,[esi].ModuleList
                    mov     ecx,[edx]
                    add     edx,4
                    ASSUME edx:PTR NET_mLIST_ELEM
@_2:
                    mov     eax,[edx].mType
                    mov     ebx,[edx].mAddr

                    .if ah == _INPUT_
                        .if eax == _CONST
                            ASSUME  ebx:PTR INPUT_CLASS
                            MOVR8 [ebx].smp,edi
                            add     edi,8                   ; sizeof INPUT_CLASS.smp
                            ASSUME  ebx:NOTHING
                        .elseif eax == _POT
                            ASSUME  ebx:PTR POT_CLASS
                            MOVR8 [ebx].smp,edi
                            MOVR8 [ebx].min,edi+8           ; sizeof POT_CLASS.smp
                            MOVR8 [ebx].max,edi+16          ; sizeof POT_CLASS.min
                            MOVR8 [ebx].stepsize,edi+24     ; sizeof POT_CLASS.max
                            add     edi,32
                            ASSUME  ebx:NOTHING
                        .endif

                    .elseif  ah == _OSC_
                        ASSUME ebx:PTR OSC1_CLASS

                        cmp     DWORD PTR [edi],NULL
                        jz  @_22

                        push    edx
                        push    ecx
                        push    esi

                        mov     edx,[esi].ModuleList
                        add     edx,4
@_21:
                        xor     ecx,ecx
                        mov     cx,[edi+1]                  ; connected module's id
                        sub     ecx,1
                        shl     ecx,4                       ; * sizeof NET_mLIST_ELEM
                        mov     esi,[edx+ecx].mAddr
                        ASSUME esi:PTR MDL_CLASS
                        mov     al,[edi+3]                  ; connected module's output
                        push    eax
                        call    [esi].GetOutput             ; get output's address
                        push    eax
                        mov     ecx,[edi]
                        mov     esi,ebx
                        push    ecx
                        call    [esi].Connect
                        add     edi,4
                        cmp     DWORD PTR [edi],NULL
                        jnz  @_21

                        pop     esi
                        pop     ecx
                        pop     edx

@_22:
                        add     edi,4
                    .elseif  ah == _OSC2_
                    .else
                    .endif

                    add     edx,sizeof NET_mLIST_ELEM
                    sub     ecx,1
                    jnz @_2
                    

                    ASSUME edx:NOTHING
@_END:
                    ret
                    ASSUME esi:NOTHING
NET_Build           ENDP
; 
;
; METHOD: Export network
;
; 
NET_Export          PROC lpData:DWORD
align 4
                    ASSUME esi:PTR NET_CLASS

                    ret
                    ASSUME esi:NOTHING
NET_Export          ENDP
; 
end